#!/usr/bin/env python

"""████
 ██    ██    Datature
   ██  ██    Powering Breakthrough AI
     ██

@File    :   __init__.py
@Author  :   Wei Loon Cheng
@Contact :   developers@datature.io
@License :   Apache License 2.0
@Desc    :   Datature Vi SDK NIM deployment module.
"""

from vi.deployment.nim.config import NIMConfig, NIMSamplingParams
from vi.deployment.nim.deployer import NIMDeployer, NIMDeploymentResult
from vi.deployment.nim.exceptions import (
    ContainerExistsError,
    InvalidConfigError,
    ModelIncompatibilityError,
    NIMDeploymentError,
    ServiceNotReadyError,
)
from vi.deployment.nim.predictor import NIMPredictor
from vi.deployment.nim.utils import encode_media_file, is_video_file

__all__ = [
    "NIMDeployer",
    "NIMConfig",
    "NIMSamplingParams",
    "NIMDeploymentResult",
    "NIMDeploymentError",
    "InvalidConfigError",
    "ContainerExistsError",
    "ServiceNotReadyError",
    "ModelIncompatibilityError",
    "NIMPredictor",
    "is_video_file",
    "encode_media_file",
]
