#!/usr/bin/env python

"""████
 ██    ██    Datature
   ██  ██    Powering Breakthrough AI
     ██

@File    :   consts.py
@Author  :   Wei Loon Cheng
@Contact :   developers@datature.io
@License :   Apache License 2.0
@Desc    :   Datature Vi SDK client constants module.
"""

DATATURE_VI_API_ENDPOINT = "https://api.vi.datature.com/"

REQUEST_CONNECT_TIMEOUT_SECONDS = 60
REQUEST_READ_TIMEOUT_SECONDS = 300
REQUEST_WRITE_TIMEOUT_SECONDS = 300
REQUEST_POOL_TIMEOUT_SECONDS = 60
REQUEST_MAX_RETRIES = 3
